@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/v5.3/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/internal/Corefinement/face_graph_utils.h $
-// $Id: face_graph_utils.h 79ba1af 2021-03-24T15:05:24+01:00 Laurent Rineau
+// $URL: https://github.com/CGAL/cgal/blob/v5.4/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/internal/Corefinement/face_graph_utils.h $
+// $Id: face_graph_utils.h 8ca5971 2021-12-07T11:30:34+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -417,6 +417,7 @@ struct Default_visitor{
   typedef boost::graph_traits<TriangleMesh> GT;
   typedef typename GT::face_descriptor face_descriptor;
   typedef typename GT::halfedge_descriptor halfedge_descriptor;
+  typedef typename GT::vertex_descriptor vertex_descriptor;
 // face visitor functions
   void before_subface_creations(face_descriptor /*f_old*/,TriangleMesh&){}
   void after_subface_creations(TriangleMesh&){}
@@ -443,19 +444,25 @@ struct Default_visitor{
   void intersection_edge_copy(halfedge_descriptor /* h_old1 */, const TriangleMesh& /* tm1 */,
                               halfedge_descriptor /* h_old2 */, const TriangleMesh& /* tm2 */,
                               halfedge_descriptor /* h_new */,  TriangleMesh& /* tm_new */){}
+// vertex visitor functions
+  void new_vertex_added(std::size_t /* node_id */,
+                        vertex_descriptor /* vh */,
+                        const TriangleMesh& /* tm */){}
 
-
+  void intersection_point_detected(std::size_t /* node_id */,
+                                   int /* sdim */,
+                                   halfedge_descriptor /* principal_edge */,
+                                   halfedge_descriptor /* additional_edge */,
+                                   const TriangleMesh& /* tm1 */,
+                                   const TriangleMesh& /* tm2 */,
+                                   bool /* is_target_coplanar */,
+                                   bool /* is_source_coplanar */) {}
+  void before_vertex_copy(vertex_descriptor /*v_src*/, const TriangleMesh& /*tm_src*/, TriangleMesh& /*tm_tgt*/){}
+  void after_vertex_copy(vertex_descriptor /*v_src*/, const TriangleMesh& /*tm_src*/,
+                         vertex_descriptor /* v_tgt */, TriangleMesh& /*tm_tgt*/){}
 // calls commented in the code and probably incomplete due to the migration
 // see NODE_VISITOR_TAG
 /*
-  void new_node_added(  std::size_t node_id,
-                        Intersection_type type,
-                        halfedge_descriptor principal_edge,
-                        halfedge_descriptor additional_edge,
-                        bool is_target_coplanar,
-                        bool is_source_coplanar)
-  {}
-
   // autorefinement only
   void new_node_added_triple_face(std::size_t node_id,
                                   face_descriptor f1,
@@ -463,10 +470,6 @@ struct Default_visitor{
                                   face_descriptor f3,
                                   const TriangleMesh& tm)
   {}
-
-  void new_vertex_added(std::size_tnode_id,
-                        vertex_descriptor vh,
-                        TriangleMesh& tm){}
 */
 };
 
@@ -504,10 +507,10 @@ triangulate_a_face(
   for(Node_id node_id : node_ids)
   {
     vertex_descriptor v=add_vertex(tm);
-//    user_visitor.new_vertex_added(node_id, v, tm); // NODE_VISITOR_TAG
     nodes.call_put(vpm, v, node_id, tm);
     // register the new vertex in the output builder
     output_builder.set_vertex_id(v, node_id, tm);
+    user_visitor.new_vertex_added(node_id, v, tm);
 
     CGAL_assertion(node_id_to_vertex.size()>node_id);
     node_id_to_vertex[node_id]=v;
@@ -861,9 +864,11 @@ void import_polyline(
 
   if( insert_res.second )
   {
+    user_visitor.before_vertex_copy(source(h1,pm1), pm1, output);
     src = add_vertex(output);
     set_halfedge(src, opposite(h_out,output),output);
     put(vpm_out, src, get(vpm1, source(h1,pm1)));
+    user_visitor.after_vertex_copy(source(h1,pm1), pm1, src, output);
     insert_res.first->second = src;
   }
   else
@@ -877,18 +882,22 @@ void import_polyline(
       pm1_to_output_vertices.insert( std::make_pair( target(h1,pm1), tgt ) );
     if( insert_res.second )
     {
+      user_visitor.before_vertex_copy(target(h1,pm1), pm1, output);
       tgt = add_vertex(output);
       set_halfedge(tgt, h_out, output);
       put(vpm_out, tgt, get(vpm1, target(h1,pm1)));
+      user_visitor.after_vertex_copy(target(h1,pm1), pm1, tgt, output);
       insert_res.first->second = tgt;
     }
     else
       tgt = insert_res.first->second;
   }
   else{
+    user_visitor.before_vertex_copy(target(h1,pm1), pm1, output);
     tgt = add_vertex(output);
     set_halfedge(tgt, h_out, output);
     put(vpm_out, tgt, get(vpm1, target(h1,pm1)));
+    user_visitor.after_vertex_copy(target(h1,pm1), pm1, tgt, output);
   }
 
   //update source and target vertex of the edge created
@@ -922,19 +931,23 @@ void import_polyline(
     //if this is the final segment, only create a target vertex if it does not exist
     if (i+1!=nb_segments)
     {
+      user_visitor.before_vertex_copy(target(h1,pm1), pm1, output);
       tgt=add_vertex(output);
       set_halfedge(tgt, h_out, output);
       put(vpm_out, tgt, get(vpm1, target(h1,pm1)));
+      user_visitor.after_vertex_copy(target(h1,pm1), pm1, tgt, output);
     }
     else{
       std::pair< typename VertexMap::iterator, bool > insert_res =
         pm1_to_output_vertices.insert(std::make_pair(target(h1,pm1), tgt));
       if (insert_res.second)
       {
+        user_visitor.before_vertex_copy(target(h1,pm1), pm1, output);
         tgt=add_vertex(output);
         set_halfedge(tgt, h_out, output);
         put(vpm_out, tgt, get(vpm1, target(h1,pm1)));
         insert_res.first->second = tgt;
+        user_visitor.after_vertex_copy(target(h1,pm1), pm1, tgt, output);
       }
       else
         tgt = insert_res.first->second;
@@ -1073,9 +1086,17 @@ void append_patches_to_triangle_mesh(
   Triangle_mesh_extension_helper<TriangleMesh, reverse_patch_orientation>
     helper(tm_to_output_edges, tm, output);
 
+  std::vector<std::size_t> ids_of_patches_to_append;
+  ids_of_patches_to_append.reserve(patches_to_append.count());
   for (std::size_t i=patches_to_append.find_first();
                    i < patches_to_append.npos;
                    i = patches_to_append.find_next(i))
+  {
+    ids_of_patches_to_append.push_back(i);
+  }
+
+  std::vector<halfedge_descriptor> interior_vertex_halfedges;
+  for (std::size_t i : ids_of_patches_to_append)
   {
     #ifdef CGAL_COREFINEMENT_POLYHEDRA_DEBUG
     #warning the size of tm_to_output_edges will increase at each step \
@@ -1086,8 +1107,6 @@ void append_patches_to_triangle_mesh(
 
     Patch_description<TriangleMesh>& patch = patches[i];
 
-    std::vector<halfedge_descriptor> interior_vertex_halfedges;
-
     //insert interior halfedges and create interior vertices
     for(halfedge_descriptor h : patch.interior_edges)
     {
@@ -1112,23 +1131,32 @@ void append_patches_to_triangle_mesh(
       if (  halfedge(target(h,tm),tm)==h &&
             patch.interior_vertices.count(target(h, tm)) )
       {
+        user_visitor.before_vertex_copy(target(h,tm), tm, output);
         vertex_descriptor v = add_vertex(output);
         set_halfedge(v, new_h, output);
         set_target(new_h, v, output);
         put(vpm_out, v, get(vpm_tm, target(h, tm) ) );
+        user_visitor.after_vertex_copy(target(h,tm), tm, v, output);
         interior_vertex_halfedges.push_back( new_h );
       }
       if (  halfedge(source(h,tm),tm)==opposite(h,tm) &&
             patch.interior_vertices.count(source(h,tm)) )
       {
+        user_visitor.before_vertex_copy(source(h,tm), tm, output);
         vertex_descriptor v = add_vertex(output);
         halfedge_descriptor new_h_opp = opposite(new_h, output);
         set_halfedge(v, new_h_opp, output);
         set_target(new_h_opp, v, output);
         put(vpm_out, v, get(vpm_tm, source(h, tm) ) );
+        user_visitor.after_vertex_copy(source(h,tm), tm, v, output);
         interior_vertex_halfedges.push_back( new_h_opp );
       }
     }
+  }
+
+  for (std::size_t i : ids_of_patches_to_append)
+  {
+    Patch_description<TriangleMesh>& patch = patches[i];
 
     //create faces and connect halfedges
     for(face_descriptor f : patch.faces)
@@ -1147,20 +1175,24 @@ void append_patches_to_triangle_mesh(
         set_face(hedges[i], new_f, output);
       }
     }
+  }
 
-    // handle interior edges that are on the border of the mesh:
-    // they do not have a prev/next pointer set since only the pointers
-    // of patch interior halfedges part a face have been. In the following
-    // (i) we set the next/prev pointer around interior vertices on the mesh
-    // boundary and (ii) we collect interior mesh border halfedges incident to
-    // a patch border vertex and set their next/prev pointer (possibly of
-    // another patch)
+  // handle interior edges that are on the border of the mesh:
+  // they do not have a prev/next pointer set since only the pointers
+  // of patch interior halfedges part a face have been. In the following
+  // (i) we set the next/prev pointer around interior vertices on the mesh
+  // boundary and (ii) we collect interior mesh border halfedges incident to
+  // a patch border vertex and set their next/prev pointer (possibly of
+  // another patch)
 
-    // Containers used for step (ii) for collecting mesh border halfedges
-    // with source/target on an intersection polyline that needs it prev/next
-    // pointer to be set
-    std::vector<halfedge_descriptor> border_halfedges_source_to_link;
-    std::vector<halfedge_descriptor> border_halfedges_target_to_link;
+  // Containers used for step (ii) for collecting mesh border halfedges
+  // with source/target on an intersection polyline that needs it prev/next
+  // pointer to be set
+  std::vector<halfedge_descriptor> border_halfedges_source_to_link;
+  std::vector<halfedge_descriptor> border_halfedges_target_to_link;
+  for (std::size_t i : ids_of_patches_to_append)
+  {
+    Patch_description<TriangleMesh>& patch = patches[i];
     for(halfedge_descriptor h : patch.interior_edges)
       if (is_border_edge(h,tm))
       {
@@ -1187,41 +1219,46 @@ void append_patches_to_triangle_mesh(
         CGAL_assertion(is_border(h_out,output) && is_border(h_out_next,output));
         set_next(h_out, h_out_next, output);
       }
-    // now the step (ii) we look for the candidate halfedge by turning around
-    // the vertex in the direction of the interior of the patch
-    for(halfedge_descriptor h_out : border_halfedges_target_to_link)
-    {
-      halfedge_descriptor candidate =
-        opposite(prev(opposite(h_out, output), output), output);
-      CGAL_assertion_code(halfedge_descriptor start=candidate);
-      while (!is_border(candidate, output)){
-        candidate=opposite(prev(candidate, output), output);
-        CGAL_assertion(candidate!=start);
-      }
-      set_next(h_out, candidate, output);
-    }
-    for(halfedge_descriptor h_out : border_halfedges_source_to_link)
-    {
-      halfedge_descriptor candidate =
-      opposite(next(opposite(h_out, output), output), output);
-      while (!is_border(candidate, output))
-        candidate = opposite(next(candidate, output), output);
-      set_next(candidate, h_out, output);
-    }
+  }
 
-    // For all interior vertices, update the vertex pointer
-    // of all but the vertex halfedge
-    for(halfedge_descriptor h_out : interior_vertex_halfedges)
-    {
-      vertex_descriptor v = target(h_out, output);
-      halfedge_descriptor next_around_vertex=h_out;
-      do{
-        CGAL_assertion(next(next_around_vertex, output) != GT::null_halfedge());
-        next_around_vertex=opposite(next(next_around_vertex, output), output);
-        set_target(next_around_vertex, v, output);
-      }while(h_out != next_around_vertex);
+  // now the step (ii) we look for the candidate halfedge by turning around
+  // the vertex in the direction of the interior of the patch
+  for(halfedge_descriptor h_out : border_halfedges_target_to_link)
+  {
+    halfedge_descriptor candidate =
+      opposite(prev(opposite(h_out, output), output), output);
+    CGAL_assertion_code(halfedge_descriptor start=candidate);
+    while (!is_border(candidate, output)){
+      candidate=opposite(prev(candidate, output), output);
+      CGAL_assertion(candidate!=start);
     }
+    set_next(h_out, candidate, output);
+  }
+  for(halfedge_descriptor h_out : border_halfedges_source_to_link)
+  {
+    halfedge_descriptor candidate =
+    opposite(next(opposite(h_out, output), output), output);
+    while (!is_border(candidate, output))
+      candidate = opposite(next(candidate, output), output);
+    set_next(candidate, h_out, output);
+  }
 
+  // For all interior vertices, update the vertex pointer
+  // of all but the vertex halfedge
+  for(halfedge_descriptor h_out : interior_vertex_halfedges)
+  {
+    vertex_descriptor v = target(h_out, output);
+    halfedge_descriptor next_around_vertex=h_out;
+    do{
+      CGAL_assertion(next(next_around_vertex, output) != GT::null_halfedge());
+      next_around_vertex=opposite(next(next_around_vertex, output), output);
+      set_target(next_around_vertex, v, output);
+    }while(h_out != next_around_vertex);
+  }
+
+  for (std::size_t i : ids_of_patches_to_append)
+  {
+    Patch_description<TriangleMesh>& patch = patches[i];
     // For all patch boundary vertices, update the vertex pointer
     // of all but the vertex halfedge
     for(halfedge_descriptor h : patch.shared_edges)
